/************************************************************************************
managementofareamapirs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//***************************************************************************************//
//* ManagementOfAreaMap.cpp						     2010.03.06 *//
//*											*//
//* コンフィギュレーション								*//
//*	01_robotmapaddress	: robotマップアドレス					*//
//*	02_gui			: GUI動作						*//
//*				  [off, on]						*//
//*	03_guiaddress		: GUIプログラムアドレス					*//
//*	04_ip			: IPアドレス						*//
//*	05_sendport		: 送信ポート番号					*//
//*	06_receiveport		: 受信ポート番号					*//
//***************************************************************************************//


// -*- C++ -*-
/*!
 * @file  ManagementOfAreaMap.cpp
 * @brief Management of Area Map
 * $Date$
 *
 * $Id$
 */

#include "ManagementOfAreaMap.h"


//-- [変数宣言] ---------------------------------------------------------------------------------//
int wait_flag = 1;											// フラグ格納変数 [読み込み用]
//-----------------------------------------------------------------------------------------------//


// Module specification
// <rtc-template block="module_spec">
static const char* managementofareamap_spec[] =
  {
    "implementation_id", "ManagementOfAreaMap",
    "type_name",         "ManagementOfAreaMap",
    "description",       "Management of Area Map",
    "version",           "0.1.0",
    "vendor",            "IRS",
    "category",          "Map",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.01_robotmapaddress", "../mapdata/robot/",
    "conf.default.02_gui", "off",
    "conf.default.03_guiaddress", "../gui/managementofareamap",
    "conf.default.04_ip", "127.0.0.1",
    "conf.default.05_sendport", "10004",
    "conf.default.06_receiveport", "10003",

    ""
  };
// </rtc-template>

ManagementOfAreaMap::ManagementOfAreaMap(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_MapOrderIn("MapOrder", m_MapOrder),
    m_MapDataOut("MapData", m_MapData),
    m_ArcNodeOut("ArcNode", m_ArcNode),
    m_LandmarkOut("Landmark", m_Landmark),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("MapOrder", m_MapOrderIn);
  
  // Set OutPort buffer
  registerOutPort("MapData", m_MapDataOut);
  registerOutPort("ArcNode", m_ArcNodeOut);
  registerOutPort("Landmark", m_LandmarkOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

ManagementOfAreaMap::~ManagementOfAreaMap()
{
}


RTC::ReturnCode_t ManagementOfAreaMap::onInitialize()
{
	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	bindParameter("01_robotmapaddress", m_01_robotmapaddress, "../mapdata/robot/");
	bindParameter("02_gui", m_02_gui, "on");
	bindParameter("03_guiaddress", m_03_guiaddress, "../gui/managementofareamap");
	bindParameter("04_ip", m_04_ip, "127.0.0.1");
	bindParameter("05_sendport", m_05_sendport, "10004");
	bindParameter("06_receiveport", m_06_receiveport, "10003");

	// </rtc-template>


	//-- [初期化] ---------------------------------------------------------------------------//

	m_MapData.location.length(DATASIZE);								// メモリ確保 [location]
	m_MapData.mapset.length(DATASIZE);								// メモリ確保 [mapset]
	m_MapData.type.length(DATASIZE);								// メモリ確保 [type]
	m_MapData.explanation.length(DATASIZE);								// メモリ確保 [explanation]

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t ManagementOfAreaMap::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManagementOfAreaMap::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManagementOfAreaMap::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t ManagementOfAreaMap::onActivated(RTC::UniqueId ec_id)
{


	//-- [初期化] ---------------------------------------------------------------------------//

	wait_flag    = 1;										// フラグ初期化 [読み取り用]
	gui_flag     = 0;										// GUI動作フラグ初期化
	sec          = 0;										// タイムスタンプ初期化 [sec]
	nsec         = 0;										// タイムスタンプ初期化 [nsec]
	trigger      = 0;										// 動作トリガー初期化
	mode         = 0;										// アクセスモード初期化
	strcpy(location, "");										// 地域名初期化
	strcpy(mapset, "");										// 利用者マップセット初期化
	strcpy(type, "");										// データタイプ初期化
	origin_x     = 0;										// 対象原点x座標初期化
	origin_y     = 0;										// 対象原点y座標初期化
	size_x       = 0;										// 対象範囲x座標初期化
	size_y       = 0;										// 対象範囲y座標初期化
	strcpy(send_data, "");										// 送信データ初期化
	trigger_flag = 0;										// 動作トリガーフラグ初期化

	//---------------------------------------------------------------------------------------//


	//-- [出力ポート初期化] -----------------------------------------------------------------//

	// タイムスタンプ初期化
	m_MapData.tm.sec  = 0;										// 初期化 [tm]
	m_MapData.tm.nsec = 0;										// 初期化 [tm]

	// モード初期化
	m_MapData.mode = 0;										// 初期化 [mode]

	// 地域名初期化
	m_MapData.location[0] = 0x00;									// 初期化 [location]

	// 利用者マップセット名初期化
	m_MapData.mapset[0] = 0x00;									// 初期化 [mapset]

	// データタイプ初期化
	m_MapData.type[0] = 0x00;									// 初期化 [type]

	// 対象原点x座標初期化
	m_MapData.origin_x = 0;										// 初期化 [origin_x]

	// 対象原点y座標初期化
	m_MapData.origin_y = 0;										// 初期化 [origin_y]

	// 対象範囲x座標初期化
	m_MapData.size_x = 0;										// 初期化 [size_x]

	// 対象範囲y座標初期化
	m_MapData.size_y = 0;										// 初期化 [size_y]

	// 地図説明初期化
	m_MapData.explanation[0] = 0x00;								// 初期化 [explanation]

	// 地図原点x座標初期化
	m_MapData.map_origin_x = 0;									// 初期化 [map_origin_x]

	// 地図原点y座標初期化
	m_MapData.map_origin_y = 0;									// 初期化 [map_origin_y]

	// 地図サイズx座標初期化
	m_MapData.map_size_x = 0;									// 初期化 [map_size_x]

	// 地図サイズy座標初期化
	m_MapData.map_size_y = 0;									// 初期化 [map_size_y]

	// ピクセルサイズx座標初期化
	m_MapData.pixel_size_x = 0;									// 初期化 [pixel_size_x]

	// ピクセルサイズy座標初期化
	m_MapData.pixel_size_y = 0;									// 初期化 [pixel_size_y]

	// タイムスタンプ初期化
	m_ArcNode.tm.sec  = 0;										// 初期化 [tm]
	m_ArcNode.tm.nsec = 0;										// 初期化 [tm]

	//---------------------------------------------------------------------------------------//


	//-- [コンフィギュレーション読み込み] ---------------------------------------------------//

	strcpy(robotmap_addr, m_01_robotmapaddress.c_str());						// データ格納 [robotマップアドレス]
	strcpy(gui,           m_02_gui.c_str());							// データ格納 [GUI動作]
	strcpy(gui_addr,      m_03_guiaddress.c_str());							// データ格納 [GUIプログラムアドレス]
	strcpy(ip,            m_04_ip.c_str());								// データ格納 [IPアドレス]
	strcpy(send_port,     m_05_sendport.c_str());							// データ格納 [送信ポート番号]
	strcpy(recv_port,     m_06_receiveport.c_str());						// データ格納 [受信ポート番号]

	//---------------------------------------------------------------------------------------//


	//-- [デバッグconfiguration] ------------------------------------------------------------//

	#ifndef DEBUG_CONFIGURATION									// デバッグ表示開始

	printf("\n[configuration]\n");									// label
	printf("01_robotmapaddress = %s\n", robotmap_addr);						// robotmapaddress
	printf("02_gui             = %s\n", gui);							// gui
	printf("03_guiaddress      = %s\n", gui_addr);							// guiaddress
	printf("04_ip              = %s\n", ip);							// ip
	printf("05_sendport        = %s\n", send_port);							// sendport
	printf("06_receiveport     = %s\n\n", recv_port);						// receiveport
	fflush(stdout);											// バッファフラッシュ [printf表示用]

	#endif												// デバッグ表示終了

	//---------------------------------------------------------------------------------------//


	//-- [コンフィギュレーションチェック] ---------------------------------------------------//

	// GUI動作チェック
	if(strcmp("off", gui) == 0) gui_flag = 0;							// GUI機能OFF
	else if(strcmp("on", gui) == 0) gui_flag = 1;							// GUI機能ON
	else{

		printf("configuration error:02_gui = %s\n", gui);					// エラーメッセージ
		fflush(stdout);										// バッファフラッシュ [printf表示用]

		return RTC::RTC_ERROR;									// 戻り値 [異常終了]

	}

	//---------------------------------------------------------------------------------------//


	//-- [GUI関連] --------------------------------------------------------------------------//

	if(gui_flag == 1){										// GUI機能ON


		//-- [GUIプログラム起動] --------------------------------------------------------//

		strcpy(command, gui_addr);								// 実行プログラム
		strcat(command, " ");									// 空白
		strcat(command, ip);									// オプション [IPアドレス]
		strcat(command, " ");									// 空白
		strcat(command, recv_port);								// オプション [受信ポート番号]
		strcat(command, " ");									// 空白
		strcat(command, send_port);								// オプション [送信ポート番号]
		strcat(command, " &");									// オプション [バックグラウンド]
		system(command);									// GUIプログラム起動
		usleep(WAITTIME);									// 待機 WAITTIME [us]

		//-------------------------------------------------------------------------------//


		//-- [UDP通信関連初期化] --------------------------------------------------------//

		memset(send_data, 0, sizeof(send_data));						// 送信データ初期化
		memset(recv_data, 0, sizeof(recv_data));						// 受信データ初期化

		//-------------------------------------------------------------------------------//


		//-- [ソケット作成] -------------------------------------------------------------//
		//-										-//
		//- プロトコル・ファミリ							-//
		//- AF_UNIX, AF_LOCAL	: ローカル通信						-//
		//- AF_INET		: IPv4 インターネットプロトコル				-//
		//- AF_INET6		: IPv6 インターネットプロトコル				-//
		//- AF_IPX		: IPX-NoveIIプロトコル					-//
		//- AF_NETLINK		: カーネルユーザデバイス				-//
		//- AF_X25		: ITU-T X.25/ISO-8208プロトコル				-//
		//- AF_AX25		: アマチュア無線AX.25プロトコル				-//
		//- AF_ATMPVC		: ATMPVC						-//
		//- AF_APPLETALK	: アップルトーク					-//
		//- AF_PACKET		: 低レベルパケットインターフェイス			-//
		//-										-//
		//- 通信方式									-//
		//- SOCK_STREAM		: バイトストリーム	[信頼性有:順序保証有:双方向]	-//
		//- SOCK_DGRAM		: データグラム		[信頼性無:固定最大長]		-//
		//- SOCK_SEQPACKET	: データグラム		[信頼性有:固定最大長:双方向]	-//
		//- SOCK_RAW		: 生ネットワーク					-//
		//- SOCK_RDM		: データグラム		[信頼性有:順序保証無]		-//
		//- SOCK_PACKET		: 廃止							-//
		//-------------------------------------------------------------------------------//

		if((send_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){					// ソケット作成 [送信データ用]

			printf("socket error!!\n");							// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		if((recv_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){					// ソケット作成 [受信データ用]

			printf("socket error!!\n");							// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		//------------------------------------------------------------------------------//


		//-- [アドレス設定] ------------------------------------------------------------//

		send_addr.sin_family      = AF_INET;							// アドレスファミリ
		send_addr.sin_port        = htons(atoi(send_port));					// ポート番号 [バイトオーダ]
		send_addr.sin_addr.s_addr = inet_addr(ip);						// IPアドレス [バイトオーダ]

		recv_addr.sin_family      = AF_INET;							// アドレスファミリ
		recv_addr.sin_port        = htons(atoi(recv_port));					// ポート番号 [バイトオーダ]
		recv_addr.sin_addr.s_addr = inet_addr(ip);						// IPアドレス [バイトオーダ]

		//------------------------------------------------------------------------------//


		//-- [受信設定] ----------------------------------------------------------------//

		bind(recv_sock, (struct sockaddr *)&recv_addr, sizeof(recv_addr));			// バインド [受信データ]
		addr_size = sizeof(recv_addr);								// アドレス格納構造体サイズ格納

		//------------------------------------------------------------------------------//


		//-- [シグナルハンドル設定] ----------------------------------------------------//

		saio.sa_handler = signal_handler_IO;							// シグナルハンドル
		sigemptyset(&saio.sa_mask);								// 追加マスクシグナル
		saio.sa_flags = 0;									// 動作詳細設定フラグ
		saio.sa_restorer = NULL;								// オボソリート [= NULL]

		//------------------------------------------------------------------------------//


		//-- [シグナルアクション調査/変更] ---------------------------------------------//

		sigaction(SIGIO, &saio, NULL);								// シグナルアクション調査/変更

		//------------------------------------------------------------------------------//


		//-- [非同期通信モード設定] ----------------------------------------------------//

		fcntl(recv_sock, F_SETOWN, getpid());							// シグナル受け取り設定 [プロセス]
		fcntl(recv_sock, F_SETFL, O_NONBLOCK | FASYNC);						// 非同期通信モード設定

		//------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t ManagementOfAreaMap::onDeactivated(RTC::UniqueId ec_id)
{


	//-- [GUI関連] --------------------------------------------------------------------------//

	if(gui_flag == 1){										// GUI機能ON


		//-- [GUIプログラム終了コマンド送信] --------------------------------------------//

		strcpy(send_data, "quit");								// 送信データ設定

		if(sendto(send_sock, send_data, strlen(send_data), 0,
			(struct sockaddr *)&send_addr, sizeof(send_addr)) < 0){				// データ送信

			printf("send error!!\n");							// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		//-------------------------------------------------------------------------------//


		//-- [ソケットディスクリプタクローズ] -------------------------------------------//

		close(send_sock);									// ソケットディスクリプタクローズ
		close(recv_sock);									// ソケットディスクリプタクローズ

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t ManagementOfAreaMap::onExecute(RTC::UniqueId ec_id)
{


	//-- [データ更新] -----------------------------------------------------------------------//

	if(m_MapOrderIn.isNew()){									// 更新チェック

		m_MapOrderIn.read();									// データ読み込み


		//-- [データ格納] ---------------------------------------------------------------//

		sec     = m_MapOrder.tm.sec;								// タイムスタンプ格納 [sec]
		nsec    = m_MapOrder.tm.nsec;								// タイムスタンプ格納 [nsec]
		trigger = m_MapOrder.trigger;								// 動作トリガー格納

		if(trigger == 0) trigger_flag = 0;							// 動作トリガーフラグクリア
		else if((trigger == 1) && (trigger_flag == 0)) trigger_flag = 1;			// 動作トリガーフラグセット
		else trigger_flag = - 1;								// 動作トリガーフラグセット

		mode    = m_MapOrder.mode;								// アクセスモード格納

		for(i = 0; i < BUFSIZE; i++){

			location[i] = m_MapOrder.location[i];						// 地域名格納
			if(m_MapOrder.location[i] == 0x00) break;					// 終端文字チェック

		}

		for(i = 0; i < BUFSIZE; i++){

			mapset[i] = m_MapOrder.mapset[i];						// 利用者マップセット格納
			if(m_MapOrder.mapset[i] == 0x00) break;						// 終端文字チェック

		}

		for(i = 0; i < BUFSIZE; i++){

			type[i] = m_MapOrder.type[i];							// データタイプ格納
			if(m_MapOrder.type[i] == 0x00) break;						// 終端文字チェック

		}

		origin_x = m_MapOrder.origin_x;								// 対象原点x座標格納
		origin_y = m_MapOrder.origin_y;								// 対象原点y座標格納
		size_x   = m_MapOrder.size_x;								// 対象範囲x座標格納
		size_y   = m_MapOrder.size_y;								// 対象範囲y座標格納

		//-------------------------------------------------------------------------------//


		//-- [デバッグinport] -----------------------------------------------------------//

		#ifndef DEBUG_INPORT									// デバッグ表示開始

		printf("\n[inport]\n");									// label
		printf("timestamp = %ld.%ld\n", sec, nsec);						// timestamp
		printf("trigger   = %d\n", trigger);							// trigger
		printf("mode      = %d\n", mode);							// mode
		printf("location  = %s\n", location);							// location
		printf("mapset    = %s\n", mapset);							// mapset
		printf("type      = %s\n", type);							// type
		printf("origin_x  = %ld\n", origin_x);							// origin_x
		printf("origin_y  = %ld\n", origin_y);							// origin_y
		printf("size_x    = %ld\n", size_x);							// size_x
		printf("size_y    = %ld\n\n", size_y);							// size_y
		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-- [トリガー] -----------------------------------------------------------------//

		if(trigger_flag == 1){									// 動作トリガーチェック

			trigger_flag = - 1;								// 動作トリガーセット


			//-- [GUI関連] ----------------------------------------------------------//

			if(gui_flag == 1){								// GUI機能ON


				//-- [データ格納] -----------------------------------------------//

				sprintf(send_data, "%s,%ld.%ld,%d,%d,%s,%s,%s,%ld,%ld,%ld,%ld",
					robotmap_addr, sec, nsec, trigger, mode, location, mapset,
					type, origin_x, origin_y, size_x, size_y);			// データ格納

				//---------------------------------------------------------------//


				//-- [データ送信] -----------------------------------------------//

				if(sendto(send_sock, send_data, strlen(send_data), 0,
					(struct sockaddr *)&send_addr, sizeof(send_addr)) < 0){		// データ送信

					printf("send error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


			}

			//-----------------------------------------------------------------------//


			//-- [ラスターデータ読み込み] -------------------------------------------//

			if(((mode & 0x0002) == 0x0000) && ((mode & 0x0001) == 0x0000)){			// モードチェック


				//-- [ヘッダファイル処理] ---------------------------------------//


				//-- [ヘッダファイルアドレス設定] -------------------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000r.ini");						// ファイル名

				//---------------------------------------------------------------//


				//-- [デバッグrheaderfilename] ----------------------------------//

				#ifndef DEBUG_RHEADERFILENAME						// デバッグ表示開始

				printf("\n[headerfilename]\n");						// label
				printf("headerfilename = %s\n\n", filename);				// headerfilename
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				if(fd == NULL){

					printf("file open error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [初期化] ---------------------------------------------------//

				strcpy(label, "");							// ラベル初期化
				strcpy(explanation, "");						// 地図説明初期化
				map_origin_x = 0;							// 地図原点 [x座標]
				map_origin_y = 0;							// 地図原点 [y座標]
				map_size_x = 0;								// pixelサイズ初期化 [x座標]
				map_size_y = 0;								// pixelサイズ初期化 [y座標]

				//---------------------------------------------------------------//


				//-- [ヘッダデータ格納] -----------------------------------------//

				i = 0;									// カウンタリセット
				while((label[i] = fgetc(fd)) != ':') i++;				// ラベル格納 [explanation]
				label[i] = 0x00;							// 終端文字処理

				if(strcmp(label, "explanation") != 0){					// ラベルチェック [explanation]

					printf("headerfile:explanation error = %s\n", label);		// エラーメッセージ
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				i = 0;									// カウンタリセット
				while((explanation[i] = fgetc(fd)) != '\n') i++;			// データ格納 [explanation]
				explanation[i] = 0x00;							// 終端文字処理

				i = 0;									// カウンタリセット
				while((label[i] = fgetc(fd)) != ':') i++;				// ラベル格納 [origin_x]
				label[i] = 0x00;							// 終端文字処理

				if(strcmp(label, "origin_x") != 0){					// ラベルチェック [origin_x]

					printf("headerfile:origin_x error = %s\n", label);		// エラーメッセージ
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				i = 0;									// カウンタリセット
				while((map_data[i] = fgetc(fd)) != '\n') i++;				// データ格納 [origin_x]
				map_data[i] = 0x00;							// 終端文字処理
				map_origin_x = atol(map_data);						// 型変換 [origin_x]

				i = 0;									// カウンタリセット
				while((label[i] = fgetc(fd)) != ':') i++;				// ラベル格納 [origin_y]
				label[i] = 0x00;							// 終端文字処理

				if(strcmp(label, "origin_y") != 0){					// ラベルチェック [origin_y]

					printf("headerfile:origin_y error = %s\n", label);		// エラーメッセージ
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				i = 0;									// カウンタリセット
				while((map_data[i] = fgetc(fd)) != '\n') i++;				// データ格納 [origin_y]
				map_data[i] = 0x00;							// 終端文字処理
				map_origin_y = atol(map_data);						// 型変換 [origin_y]

				i = 0;									// カウンタリセット
				while((label[i] = fgetc(fd)) != ':') i++;				// ラベル格納 [size_x]
				label[i] = 0x00;							// 終端文字処理

				if(strcmp(label, "size_x") != 0){					// ラベルチェック [size_x]

					printf("headerfile:size_x error = %s\n", label);		// エラーメッセージ
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				i = 0;									// カウンタリセット
				while((map_data[i] = fgetc(fd)) != '\n') i++;				// データ格納 [size_x]
				map_data[i] = 0x00;							// 終端文字処理
				map_size_x = atol(map_data);						// 型変換 [size_x]

				i = 0;									// カウンタリセット
				while((label[i] = fgetc(fd)) != ':') i++;				// ラベル格納 [size_y]
				label[i] = 0x00;							// 終端文字処理

				if(strcmp(label, "size_y") != 0){					// ラベルチェック [size_y]

					printf("headerfile:size_y error = %s\n", label);		// エラーメッセージ
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				i = 0;									// カウンタリセット
				while((map_data[i] = fgetc(fd)) != '\n') i++;				// データ格納 [size_y]
				map_data[i] = 0x00;							// 終端文字処理
				map_size_y = atol(map_data);						// 型変換 [size_y]


				//-- [デバッグrheaderdata] --------------------------------------//

				#ifndef DEBUG_RHEADERDATA						// デバッグ表示開始

				printf("\n[headerdata]\n");						// label
				printf("explanation = %s\n", explanation);				// explanation
				printf("origin_x    = %ld\n", map_origin_x);				// origin_x
				printf("origin_y    = %ld\n", map_origin_y);				// origin_y
				printf("size_x      = %ld\n", map_size_x);				// size_x
				printf("size_y      = %ld\n\n", map_size_y);				// size_y
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [カテゴリ/カラーテーブルファイル処理] ----------------------//


				//-- [カテゴリファイルアドレス設定] -----------------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000r.cat");						// ファイル名

				//---------------------------------------------------------------//


				//-- [デバッグcategoryfilename] ---------------------------------//

				#ifndef DEBUG_CATEGORYFILENAME						// デバッグ表示開始

				printf("\n[categoryfilename]\n");					// label
				printf("categoryfilename = %s\n\n", filename);				// categoryfilename
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				if(fd == NULL){

					printf("file open error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [行数チェック] ---------------------------------------------//

				cat_num = 0;								// 行数初期化
				while(fgets(chline, 2 * DATASIZE, fd) != NULL) cat_num++;		// 行数カウント

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [カラーテーブルファイルアドレス設定] -----------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000r.col");						// ファイル名

				//---------------------------------------------------------------//


				//-- [デバッグcolortablefilename] -------------------------------//

				#ifndef DEBUG_COLORTABLEFILENAME					// デバッグ表示開始

				printf("\n[colortablefilename]\n");					// label
				printf("colortablefilename = %s\n\n", filename);			// colortablefilename
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				if(fd == NULL){

					printf("file open error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [行数チェック] ---------------------------------------------//

				col_num = 0;								// 行数初期化
				while(fgets(chline, 256, fd) != NULL) col_num++;			// 行数カウント

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [行数チェック] ---------------------------------------------//

				if(cat_num != col_num){

					printf("category and color table mismatch!!\n");		// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [カテゴリファイルアドレス設定] -----------------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000r.cat");						// ファイル名

				//---------------------------------------------------------------//


				//-- [メモリ確保] -----------------------------------------------//

				pcat = (Category*)calloc(cat_num, sizeof(Category));			// メモリ確保

				if(pcat == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				//---------------------------------------------------------------//


				//-- [カテゴリデータ格納] ---------------------------------------//

				for(i = 0; i < cat_num; i++)
					fscanf(fd, "%ld:%s\n", &((pcat + i)->num),
						(pcat + i)->category);					// データ格納

				//---------------------------------------------------------------//


				//-- [デバッグcategorydata] -------------------------------------//

				#ifndef DEBUG_CATEGORYDATA						// デバッグ表示開始

				printf("\n[categorydata]\n");						// label
				for(i = 0; i < cat_num; i++) printf("category_num = %ld, category = %s\n",
					(pcat + i)->num, (pcat + i)->category);				// category_num, category
				printf("\n");								// 改行
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [カラーテーブルファイルアドレス設定] -----------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000r.col");						// ファイル名

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				//---------------------------------------------------------------//


				//-- [カラーテーブルデータ格納] ---------------------------------//

				for(i = 0; i < col_num; i++){

					fscanf(fd, "%ld:%d:%d:%d\n", &num_data, &r_data,
						&g_data, &b_data);					// データ格納

					if(num_data != (pcat + i)->num) return RTC::RTC_ERROR;		// 戻り値 [異常終了]
					else (pcat + i)->rgb = r_data * 0x010000
						+ g_data * 0x000100 + b_data;				// データ格納 [RGB]

				}

				//---------------------------------------------------------------//


				//-- [デバッグcolortabledata] -----------------------------------//

				#ifndef DEBUG_COLORTABLEDATA						// デバッグ表示開始

				printf("\n[colortabledata]\n");						// label
				for(i = 0; i < col_num; i++) printf("category_num = %ld, color = %3d.%3d.%3d\n",
					(pcat + i)->num, (int)((((pcat + i)->rgb) >> 16) & 0xff) ,
					(int)((((pcat + i)->rgb) >> 8) & 0xff),
					(int)(((pcat + i)->rgb) & 0xff));				// category_num, color
				printf("\n");								// 改行
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [ビットマップファイル処理] ---------------------------------//


				//-- [ビットマップファイルアドレス設定] -------------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000r.bmp");						// ファイル名

				//---------------------------------------------------------------//


				//-- [デバッグbmpfilename] --------------------------------------//

				#ifndef DEBUG_BMPFILENAME						// デバッグ表示開始

				printf("\n[bmpfilename]\n");						// label
				printf("bmpfilename = %s\n\n", filename);				// bmpfilename
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "rb");						// ファイルオープン

				if(fd == NULL){

					printf("file open error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [画像データ格納] -------------------------------------------//

				ppic = getbitmap(fd);							// 画像データ取得

				if(ppic == NULL){

					printf("bitmap error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [読み込み範囲処理] -----------------------------------------//

				if((size_x == 0) || (size_y == 0)){					// 地図情報取得モード

						min_i = 0;						// 開始番号設定 [y座標]
						min_j = 0;						// 開始番号設定 [x座標]
						max_i = 0;						// 終了番号設定 [y座標]
						max_j = 0;						// 終了番号設定 [x座標]

				}else if(((((long)map_size_x * ppic->x) + map_origin_x) < origin_x)
					|| ((((long)map_size_y * ppic->y) + map_origin_y) < origin_y)){	// 原点対象外

						min_i = 0;						// 開始番号設定 [y座標]
						min_j = 0;						// 開始番号設定 [x座標]
						max_i = 0;						// 終了番号設定 [y座標]
						max_j = 0;						// 終了番号設定 [x座標]

				}else{									// 原点対象内

					if(origin_x < map_origin_x) min_j = 0;				// 開始番号設定 [x座標]
					else min_j = (origin_x - map_origin_x) / map_size_x;		// 開始番号設定 [x座標]

					if(origin_y < map_origin_y) min_i = 0;				// 開始番号設定 [y座標]
					else min_i = (origin_y - map_origin_y) / map_size_y;		// 開始番号設定 [y座標]

					if((origin_x + (long)size_x) < (((long)map_size_x * ppic->x) + map_origin_x))
						max_j = (origin_x + size_x - map_origin_x)
							/ map_size_x;					// 終了番号設定 [x座標]
					else max_j = ppic->x;						// 終了番号設定 [x座標]

					if((origin_y + (long)size_y) < (((long)map_size_y * ppic->y) + map_origin_y))
						max_i = (origin_y + size_y - map_origin_y)
							/ map_size_y;					// 狩猟番号設定 [y座標]
					else max_i = ppic->y;						// 終了番号設定 [y座標]

				}

				//---------------------------------------------------------------//


				//-- [地図情報格納処理] -----------------------------------------//

				m_MapData.mapdata.length((max_j - min_j) * (max_i - min_i));		// メモリ確保 [mapdata]


				//-- [デバッグbmpdata] ------------------------------------------//

				#ifndef DEBUG_BMPDATA							// デバッグ表示開始

				printf("\n[bmpdata]\n");						// label
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//

				map_num = 0;								// カウントリセット

				for(i = (ppic->y - max_i); i < (ppic->y - min_i); i++){

					for(j = min_j; j < max_j; j++){

						m_MapData.mapdata[map_num] = ppic->r[j + i * ppic->x]
								* 0x010000 + ppic->g[j + i * ppic->x]
								* 0x000100 + ppic->b[j + i * ppic->x];	// 前処理

						for(k = 0; k < (unsigned long)col_num; k++){

							if((long)((pcat + k)->rgb) == (long)((unsigned long)(ppic->r[j + i * ppic->x])
								* 0x010000 + (unsigned long)(ppic->g[j + i * ppic->x])
								* 0x000100 + (unsigned long)(ppic->b[j + i * ppic->x]))){

								m_MapData.mapdata[map_num]
									= (pcat + k)->num;		// 地図情報格納

							}

						}

						//-- [デバッグbmpdata] --------------------------//

						#ifndef DEBUG_BMPDATA					// デバッグ表示開始

printf("%ld", m_MapData.mapdata[map_num]);
//						printf("%ld", (pcat + k)->num);				// bmpdata
						fflush(stdout);						// バッファフラッシュ [printf表示用]

						#endif							// デバッグ表示終了

						//-----------------------------------------------//

						map_num++;						// カウント

					}


					//-- [デバッグbmpdata] ----------------------------------//

					#ifndef DEBUG_BMPDATA						// デバッグ表示開始

					printf("\n");							// 改行
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					#endif								// デバッグ表示終了

					//-------------------------------------------------------//


				}


				//-- [デバッグbmpdata] ------------------------------------------//

				#ifndef DEBUG_BMPDATA							// デバッグ表示開始

				printf("\n");								// 改行
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [出力データ格納] -------------------------------------------//

				m_MapData.tm.sec  = sec;						// タイムスタンプ格納 [sec]
				m_MapData.tm.nsec = nsec;						// タイムスタンプ格納 [nsec]
				m_MapData.mode    = mode;						// モード格納
				for(i = 0; i <= (unsigned long)strlen(location); i++)
					m_MapData.location[i] = location[i];				// 地域名格納
				for(i = 0; i <= (unsigned long)strlen(mapset); i++)
					m_MapData.mapset[i] = mapset[i];				// 利用者マップセット格納
				for(i = 0; i <= (unsigned long)strlen(type); i++)
					m_MapData.type[i] = type[i];					// データタイプ格納
				m_MapData.origin_x = map_origin_x + map_size_x * min_j;			// 対象原点x座標格納
				m_MapData.origin_y = map_origin_y + map_size_y * min_i;			// 対象原点y座標格納
				m_MapData.size_x   = (max_j - min_j) * map_size_x;			// 対象範囲x座標格納
				m_MapData.size_y   = (max_i - min_i) * map_size_y;			// 対象範囲y座標格納
				for(i = 0; i <= (unsigned long)strlen(explanation); i++)
					m_MapData.explanation[i] = explanation[i];			// 地図説明格納
				m_MapData.map_origin_x = map_origin_x;					// 地図原点x座標格納
				m_MapData.map_origin_y = map_origin_y;					// 地図原点y座標格納
				m_MapData.map_size_x   = ppic->x * map_size_x;				// 地図サイズx座標格納
				m_MapData.map_size_y   = ppic->y * map_size_y;				// 地図サイズy座標格納
				m_MapData.pixel_size_x = map_size_x;					// ピクセルサイズx座標格納
				m_MapData.pixel_size_y = map_size_y;					// ピクセルサイズy座標格納

				m_MapData.category.length(DATASIZE * cat_num);				// メモリ確保 [category]

				k = 0;									// カウントリセット

				for(i = 0; i < (unsigned long)cat_num; i++){

					sprintf(category, "%ld,%s,",
						(pcat + i)->num, (pcat + i)->category);			// データ一時格納 [category]
					j = 0;								// カウントリセット

					while(category[j] != 0x00){

						m_MapData.category[k] = category[j];			// カテゴリ格納
						j++;							// カウント
						k++;							// カウント

					}

				}

				m_MapData.category[k - 1] = 0x00;					// 終端文字処理

				//---------------------------------------------------------------//


				//-- [データ出力] -----------------------------------------------//

				m_MapDataOut.write();							// データ出力

				//---------------------------------------------------------------//


				//-- [メモリ解放] -----------------------------------------------//

				free(pcat);								// メモリ解放
				deletepicture(ppic);							// メモリ解放
				m_MapData.category.length(0);						// メモリ解放 [category]
				m_MapData.mapdata.length(0);						// メモリ解放 [mapdata]

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


			}

			//-----------------------------------------------------------------------//


			//-- [ランドマーク読み込み] ---------------------------------------------//

			if(((mode & 0x0002) == 0x0000) && ((mode & 0x0001) == 0x0001)
				&& (0 == strcmp("landmark", type))){						// モードチェック


				//-- [ヘッダファイル処理] ---------------------------------------//


				//-- [ヘッダファイルアドレス設定] -------------------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000v.ini");						// ファイル名

				//---------------------------------------------------------------//


				//-- [デバッグvheaderfilename] ----------------------------------//

				#ifndef DEBUG_VHEADERFILENAME						// デバッグ表示開始

				printf("\n[headerfilename]\n");						// label
				printf("headerfilename = %s\n\n", filename);				// headerfilename
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				if(fd == NULL){

					printf("file open error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [初期化] ---------------------------------------------------//

				strcpy(label, "");							// ラベル初期化
				strcpy(explanation, "");						// 地図説明初期化

				//---------------------------------------------------------------//


				//-- [ヘッダデータ格納] -----------------------------------------//

				i = 0;									// カウンタリセット
				while((label[i] = fgetc(fd)) != ':') i++;				// ラベル格納 [explanation]
				label[i] = 0x00;							// 終端文字処理

				if(strcmp(label, "explanation") != 0){					// ラベルチェック [explanation]

					printf("headerfile:explanation error = %s\n", label);		// エラーメッセージ
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				i = 0;									// カウンタリセット
				while((explanation[i] = fgetc(fd)) != '\n') i++;			// データ格納 [explanation]
				explanation[i] = 0x00;							// 終端文字処理


				//-- [デバッグvheaderdata] --------------------------------------//

				#ifndef DEBUG_VHEADERDATA						// デバッグ表示開始

				printf("\n[headerdata]\n");						// label
				printf("explanation = %s\n\n", explanation);				// explanation
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [ノードファイル処理] ---------------------------------------//


				//-- [ノードファイルアドレス設定] -------------------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000v.nod");						// ファイル名

				//---------------------------------------------------------------//


				//-- [デバッグnodefilename] -------------------------------------//

				#ifndef DEBUG_NODEFILENAME						// デバッグ表示開始

				printf("\n[nodefilename]\n");						// label
				printf("nodefilename = %s\n\n", filename);				// nodefilename
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				if(fd == NULL){

					printf("file open error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [初期化] ---------------------------------------------------//

				node_size = 0;								// ノードデータ数初期化

				//---------------------------------------------------------------//


				//-- [データ数チェック] -----------------------------------------//

				while(fgets(chline, 2 * DATASIZE, fd) != NULL) node_size++;		// 行数カウント

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [メモリ確保] -----------------------------------------------//

				px = (long*)calloc(node_size, sizeof(long));				// メモリ確保

				if(px == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				py = (long*)calloc(node_size, sizeof(long));				// メモリ確保

				if(py == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				pr = (long*)calloc(node_size, sizeof(char));				// メモリ確保

				if(pr == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				pnode = (int*)calloc(node_size, sizeof(int));				// メモリ確保

				if(pnode == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				//---------------------------------------------------------------//


				//-- [ノードデータ格納] -----------------------------------------//

				for(i = 0; i < node_size; i++)
					fscanf(fd, "%ld:%ld:%ld:%ld\n", &id, (px + i), (py + i),
						(pr + i));						// データ格納

				//---------------------------------------------------------------//


				//-- [デバッグnodedata] -----------------------------------------//

				#ifndef DEBUG_NODEDATA							// デバッグ表示開始

				printf("\n[nodedata]\n");						// label
				for(i = 0; i < node_size; i++)
					printf("id = %ld, x = %ld, y = %ld, r = %ld\n",
						i + 1, *(px + i), *(py + i), *(pr + i));		// x, y, r
				printf("\n");								// 改行
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [指定範囲選定] ---------------------------------------------//

				node_num = 0;								// ノードサイズ初期化

				for(i = 0; i < node_size; i++){

					if((origin_x <= *(px + i)) && (*(px + i) <= origin_x + (long)size_x) &&
					(origin_y <= *(py + i)) && (*(py + i) <= origin_y + (long)size_y)){

						*(pnode + i) = 1;					// 指定範囲内チェック
						node_num++;						// ノードサイズカウント

					}else *(pnode + i) = 0;						// 指定範囲外チェック

				}

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [アークファイル処理] ---------------------------------------//


				//-- [アークファイルアドレス設定] -------------------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000v.arc");						// ファイル名

				//---------------------------------------------------------------//


				//-- [デバッグarcfilename] --------------------------------------//

				#ifndef DEBUG_ARCFILENAME						// デバッグ表示開始

				printf("\n[arcfilename]\n");						// label
				printf("arcfilename = %s\n\n", filename);				// arcfilename
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				if(fd == NULL){

					printf("file open error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [初期化] ---------------------------------------------------//

				arc_size = 0;								// アークデータ数初期化

				//---------------------------------------------------------------//


				//-- [データ数チェック] -----------------------------------------//

				while(fgets(chline, 2 * DATASIZE, fd) != NULL) arc_size++;		// 行数カウント

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [メモリ確保] -----------------------------------------------//

				px2 = (long*)calloc(node_size, sizeof(long));				// メモリ確保

				if(px2 == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				py2 = (long*)calloc(node_size, sizeof(long));				// メモリ確保

				if(py2 == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				ptheta = (double*)calloc(node_size, sizeof(double));			// メモリ確保

				if(ptheta == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				pr2 = (long*)calloc(node_size, sizeof(char));				// メモリ確保

				if(pr2 == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				parc = (int*)calloc(node_size, sizeof(int));				// メモリ確保

				if(parc == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				//---------------------------------------------------------------//


				//-- [アークデータ格納] -----------------------------------------//

				for(i = 0; i < arc_size; i++)
					fscanf(fd, "%ld:%ld:%ld:%lf:%ld\n", &id, (px2 + i), (py2 + i),
						(ptheta + i), (pr2 + i));				// データ格納

				//---------------------------------------------------------------//


				//-- [デバッグarcdata] ------------------------------------------//

				#ifndef DEBUG_ARCDATA							// デバッグ表示開始

				printf("\n[arcdata]\n");						// label
				for(i = 0; i < arc_size; i++)
					printf("id = %ld, x = %ld, y = %ld, theta = %lf, r = %ld\n",
						i + 1, *(px2 + i), *(py2 + i), *(ptheta + i),
						*(pr2 + i));						// x, y, theta, r
				printf("\n");								// 改行
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [指定範囲選定] ---------------------------------------------//

				arc_num = 0;								// アークサイズ初期化

				for(i = 0; i < arc_size; i++){

					if((origin_x <= *(px2 + i)) && (*(px2 + i) <= origin_x + (long)size_x) &&
					(origin_y <= *(py2 + i)) && (*(py2 + i) <= origin_y + (long)size_y)){

						*(parc + i) = 1;					// 指定範囲内チェック
						arc_num++;						// ノードサイズカウント

					}else *(parc + i) = 0;						// 指定範囲外チェック

				}

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [出力データ格納] -------------------------------------------//


				//-- [デバッグarcnodedata] --------------------------------------//

				#ifndef DEBUG_ARCNODEDATA						// デバッグ表示開始

				printf("\n[nodedata]\n");						// label
				for(i = 0; i < node_size; i++)

					if(*(pnode + i) == 1)
						printf("id = %ld, x = %ld, y = %ld, r = %ld\n",
							i + 1, *(px + i), *(py + i), *(pr + i));	// x, y, r

				printf("\n[arcdata]\n");						// label
				for(i = 0; i < arc_size; i++)

					if(*(parc + i) == 1)
						printf("id = %ld, x = %ld, y = %ld, theta = %lf, r = %ld\n",
							i + 1, *(px2 + i), *(py2 + i), *(ptheta + i),
							*(pr2 + i));					// x, y, theta, r

				printf("\n");								// 改行

				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				m_Landmark.tm.sec  = sec;						// タイムスタンプ格納 [sec]
				m_Landmark.tm.nsec = nsec;						// タイムスタンプ格納 [nsec]

				m_Landmark.line_data.length(arc_size);					// メモリ確保 [line_data]

				for(i = 0; i < arc_size; i++){

					m_Landmark.line_data[i].x     = *(px2    + i) / 1000.0;		// x座標 [m]
					m_Landmark.line_data[i].y     = *(py2    + i) / 1000.0;		// y座標 [m]
					m_Landmark.line_data[i].theta = *(ptheta + i);			// 角度 [rad]
					m_Landmark.line_data[i].r     = *(pr2    + i) / 1000.0;		// 長さの1/2 [m]

				}

				m_Landmark.circle_data.length(node_size);				// メモリ確保 [circle_data]

				for(i = 0; i < node_size; i++){

					m_Landmark.circle_data[i].x     = *(px + i) / 1000.0;		// x座標 [m]
					m_Landmark.circle_data[i].y     = *(py + i) / 1000.0;		// y座標 [m]
					m_Landmark.circle_data[i].r     = *(pr + i) / 1000.0;		// 半径 [m]

				}

				//---------------------------------------------------------------//


				//-- [データ出力] -----------------------------------------------//

				m_LandmarkOut.write();							// データ出力

				//---------------------------------------------------------------//


				//-- [メモリ解放] -----------------------------------------------//

				free(px);								// メモリ解放
				free(py);								// メモリ解放
				free(pr);								// メモリ解放
				free(px2);								// メモリ解放
				free(py2);								// メモリ解放
				free(pr2);								// メモリ解放
				free(ptheta);								// メモリ解放
				free(pnode);								// メモリ解放
				free(parc);								// メモリ解放
				m_Landmark.line_data.length(0);						// メモリ解放 [line_data]
				m_Landmark.circle_data.length(0);					// メモリ解放 [circle_data]

				//---------------------------------------------------------------//


			}

			//-----------------------------------------------------------------------//


			//-- [ベクターデータ読み込み] -------------------------------------------//

			if(((mode & 0x0002) == 0x0000) && ((mode & 0x0001) == 0x0001)
				&& (0 != strcmp("landmark", type))){					// モードチェック


				//-- [ヘッダファイル処理] ---------------------------------------//


				//-- [ヘッダファイルアドレス設定] -------------------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000v.ini");						// ファイル名

				//---------------------------------------------------------------//


				//-- [デバッグvheaderfilename] ----------------------------------//

				#ifndef DEBUG_VHEADERFILENAME						// デバッグ表示開始

				printf("\n[headerfilename]\n");						// label
				printf("headerfilename = %s\n\n", filename);				// headerfilename
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				if(fd == NULL){

					printf("file open error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [初期化] ---------------------------------------------------//

				strcpy(label, "");							// ラベル初期化
				strcpy(explanation, "");						// 地図説明初期化

				//---------------------------------------------------------------//


				//-- [ヘッダデータ格納] -----------------------------------------//

				i = 0;									// カウンタリセット
				while((label[i] = fgetc(fd)) != ':') i++;				// ラベル格納 [explanation]
				label[i] = 0x00;							// 終端文字処理

				if(strcmp(label, "explanation") != 0){					// ラベルチェック [explanation]

					printf("headerfile:explanation error = %s\n", label);		// エラーメッセージ
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				i = 0;									// カウンタリセット
				while((explanation[i] = fgetc(fd)) != '\n') i++;			// データ格納 [explanation]
				explanation[i] = 0x00;							// 終端文字処理


				//-- [デバッグvheaderdata] --------------------------------------//

				#ifndef DEBUG_VHEADERDATA						// デバッグ表示開始

				printf("\n[headerdata]\n");						// label
				printf("explanation = %s\n\n", explanation);				// explanation
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [ノードファイル処理] ---------------------------------------//


				//-- [ノードファイルアドレス設定] -------------------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000v.nod");						// ファイル名

				//---------------------------------------------------------------//


				//-- [デバッグnodefilename] -------------------------------------//

				#ifndef DEBUG_NODEFILENAME						// デバッグ表示開始

				printf("\n[nodefilename]\n");						// label
				printf("nodefilename = %s\n\n", filename);				// nodefilename
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				if(fd == NULL){

					printf("file open error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [初期化] ---------------------------------------------------//

				node_size = 0;								// ノードデータ数初期化

				//---------------------------------------------------------------//


				//-- [データ数チェック] -----------------------------------------//

				while(fgets(chline, 2 * DATASIZE, fd) != NULL) node_size++;		// 行数カウント

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [メモリ確保] -----------------------------------------------//

				px = (long*)calloc(node_size, sizeof(long));				// メモリ確保

				if(px == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				py = (long*)calloc(node_size, sizeof(long));				// メモリ確保

				if(py == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				ptheta = (double*)calloc(node_size, sizeof(double));			// メモリ確保

				if(ptheta == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				pexplanation = (char*)calloc(node_size * DATASIZE, sizeof(char));	// メモリ確保

				if(pexplanation == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				pnode = (int*)calloc(node_size, sizeof(int));				// メモリ確保

				if(pnode == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				//---------------------------------------------------------------//


				//-- [ノードデータ格納] -----------------------------------------//

				for(i = 0; i < node_size; i++)
					fscanf(fd, "%ld:%ld:%ld:%lf:%s\n", &id, (px + i), (py + i),
						(ptheta + i), (pexplanation + i * DATASIZE));		// データ格納

				//---------------------------------------------------------------//


				//-- [デバッグnodedata] -----------------------------------------//

				#ifndef DEBUG_NODEDATA							// デバッグ表示開始

				printf("\n[nodedata]\n");						// label
				for(i = 0; i < node_size; i++)
					printf("id = %ld, x = %ld, y = %ld, theta = %lf, explanation = %s\n",
						i + 1, *(px + i), *(py + i), *(ptheta + i),
						(pexplanation + i * DATASIZE));				// x, y, theta, explanation
				printf("\n");								// 改行
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [指定範囲選定] ---------------------------------------------//

				node_num = 0;								// ノードサイズ初期化

				for(i = 0; i < node_size; i++){

					if((origin_x <= *(px + i)) && (*(px + i) <= origin_x + (long)size_x) &&
					(origin_y <= *(py + i)) && (*(py + i) <= origin_y + (long)size_y)){

						*(pnode + i) = 1;					// 指定範囲内チェック
						node_num++;						// ノードサイズカウント

					}else *(pnode + i) = 0;						// 指定範囲外チェック

				}

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [アークファイル処理] ---------------------------------------//


				//-- [アークファイルアドレス設定] -------------------------------//

				strcpy(filename, robotmap_addr);					// robotマップアドレス
				strcat(filename, location);						// 地域名
				strcat(filename, "/");							// スラッシュ
				strcat(filename, mapset);						// 利用者マップセット
				strcat(filename, "/");							// スラッシュ
				strcat(filename, type);							// データタイプ
				strcat(filename, "0000v.arc");						// ファイル名

				//---------------------------------------------------------------//


				//-- [デバッグarcfilename] --------------------------------------//

				#ifndef DEBUG_ARCFILENAME						// デバッグ表示開始

				printf("\n[arcfilename]\n");						// label
				printf("arcfilename = %s\n\n", filename);				// arcfilename
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				if(fd == NULL){

					printf("file open error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [初期化] ---------------------------------------------------//

				arc_size = 0;								// アークデータ数初期化

				//---------------------------------------------------------------//


				//-- [データ数チェック] -----------------------------------------//

				while(fgets(chline, 2 * DATASIZE, fd) != NULL) arc_size++;		// 行数カウント

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [メモリ確保] -----------------------------------------------//

				pstartid = (unsigned long*)calloc(arc_size, sizeof(unsigned long));	// メモリ確保

				if(pstartid == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				pendid = (unsigned long*)calloc(arc_size, sizeof(unsigned long));	// メモリ確保

				if(pendid == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				pcost = (double*)calloc(arc_size, sizeof(double));			// メモリ確保

				if(pcost == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				parc = (int*)calloc(arc_size, sizeof(int));				// メモリ確保

				if(parc == NULL){

					printf("memory securing error!!\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				//---------------------------------------------------------------//


				//-- [ファイルオープン] -----------------------------------------//

				fd = fopen(filename, "r");						// ファイルオープン

				//---------------------------------------------------------------//


				//-- [アークデータ格納] -----------------------------------------//

				for(i = 0; i < arc_size; i++)
					fscanf(fd, "%ld:%ld:%lf\n", (pstartid + i), (pendid + i),
						(pcost + i));						// データ格納

				//---------------------------------------------------------------//


				//-- [デバッグarcdata] ------------------------------------------//

				#ifndef DEBUG_ARCDATA							// デバッグ表示開始

				printf("\n[arcdata]\n");						// label
				for(i = 0; i < arc_size; i++)
					printf("start_id = %ld, end_id = %ld, cost = %lf\n",
					*(pstartid + i), *(pendid + i), *(pcost + i));			// startid, endid ,cost
				printf("\n");								// 改行
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				//-- [ファイルクローズ] -----------------------------------------//

				fclose(fd);								// ファイルクローズ

				//---------------------------------------------------------------//


				//-- [指定範囲選定] ---------------------------------------------//

				arc_num = 0;								// アークサイズ初期化

				for(i = 0; i < arc_size; i++){

					if((*(pnode + *(pstartid + i) - 1) == 1) &&
						(*(pnode + *(pendid + i) - 1) == 1)){

						*(parc + i) = 1;					// 指定範囲内チェック
						arc_num++;						// アークサイズカウント

					}else *(parc + i) = 0;						// 指定範囲外チェック

				}

				//---------------------------------------------------------------//


				//---------------------------------------------------------------//


				//-- [出力データ格納] -------------------------------------------//


				//-- [デバッグarcnodedata] --------------------------------------//

				#ifndef DEBUG_ARCNODEDATA						// デバッグ表示開始

				printf("\n[nodedata]\n");						// label
				for(i = 0; i < node_size; i++)

					if(*(pnode + i) == 1)
						printf("id = %ld, x = %ld, y = %ld, theta = %lf, explanation = %s\n",
							i + 1, *(px + i), *(py + i), *(ptheta + i),
							(pexplanation + i * DATASIZE));			// x, y, theta, explanation

				printf("\n[arcdata]\n");						// label
				for(i = 0; i < arc_size; i++)

					if(*(parc + i) == 1)
						printf("start_id = %ld, end_id = %ld, cost = %lf\n",
						*(pstartid + i), *(pendid + i), *(pcost + i));		// startid, endid ,cost

				printf("\n");								// 改行

				fflush(stdout);								// バッファフラッシュ [printf表示用]

				#endif									// デバッグ表示終了

				//---------------------------------------------------------------//


				m_ArcNode.tm.sec  = sec;						// タイムスタンプ格納 [sec]
				m_ArcNode.tm.nsec = nsec;						// タイムスタンプ格納 [nsec]

				m_ArcNode.arc_data.length(arc_size);					// メモリ確保 [arc_data]

				for(i = 0; i < arc_size; i++){

					m_ArcNode.arc_data[i].start_id = *(pstartid + i);		// 接続開始ノードID
					m_ArcNode.arc_data[i].end_id   = *(pendid + i);			// 接続終了ノードID
					m_ArcNode.arc_data[i].cost     = *(pcost + i);			// コスト

				}

				m_ArcNode.node_data.length(node_size);					// メモリ確保 [node_data]

				for(i = 0; i < node_size; i++){

					m_ArcNode.node_data[i].id    = i + 1;				// ノードID
					m_ArcNode.node_data[i].x     = *(px + i) / 1000.0;		// x座標 [m]
					m_ArcNode.node_data[i].y     = *(py + i) / 1000.0;		// y座標 [m]
					m_ArcNode.node_data[i].theta = *(ptheta + i);			// 方向 [rad]

					m_ArcNode.node_data[i].explanation.length(DATASIZE);		// メモリ確保 [explanation]

					for(j = 0; j < DATASIZE; j++){

						m_ArcNode.node_data[i].explanation[j] =
							*(pexplanation + i * DATASIZE + j);		// ノード説明

					}

				}

				//---------------------------------------------------------------//


				//-- [データ出力] -----------------------------------------------//

				m_ArcNodeOut.write();							// データ出力

				//---------------------------------------------------------------//


				//-- [メモリ解放] -----------------------------------------------//

				free(px);								// メモリ解放
				free(py);								// メモリ解放
				free(ptheta);								// メモリ解放
				free(pexplanation);							// メモリ解放
				free(pnode);								// メモリ解放
				free(pstartid);								// メモリ解放
				free(pendid);								// メモリ解放
				free(pcost);								// メモリ解放
				free(parc);								// メモリ解放
				m_ArcNode.arc_data.length(0);						// メモリ解放 [arc_data]
				for(i = 0; i < node_size; i++)
					m_ArcNode.node_data[i].explanation.length(0);			// メモリ解放 [explanation]
				m_ArcNode.node_data.length(0);						// メモリ解放 [node_data]

				//---------------------------------------------------------------//


			}

			//-----------------------------------------------------------------------//


		}

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	//-- [GUI関連] --------------------------------------------------------------------------//

	if(gui_flag == 1){										// GUI機能ON


		//-- [データ受信] ---------------------------------------------------------------//

		if(wait_flag == 0){

			// データ受信
			recv_len = recvfrom(recv_sock, recv_data, sizeof(recv_data), 0,
				(struct sockaddr *)&recv_addr, &addr_size);				// データ受信

			if(recv_len < 0){								// 受信チェック

				printf("receive error!!\n");						// メッセージ表示
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				return RTC::RTC_ERROR;							// 戻り値 [異常終了]

			}


			//-- [デバッグrecv] -----------------------------------------------------//

			#ifndef DEBUG_RECV								// デバッグ表示開始

			printf("\n[recv]\n");								// label
			printf("time      = %lf\n", gettimeofday_sec());				// time
			printf("recv data = %s\n\n", recv_data);					// recv_data
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			#endif										// デバッグ表示終了

			//-----------------------------------------------------------------------//


			memset(recv_data, 0, sizeof(recv_data));					// バッファクリア

			wait_flag = 1;									// フラグセット

		}

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t ManagementOfAreaMap::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManagementOfAreaMap::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManagementOfAreaMap::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManagementOfAreaMap::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManagementOfAreaMap::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{

  void ManagementOfAreaMapInit(RTC::Manager* manager)
  {
    RTC::Properties profile(managementofareamap_spec);
    manager->registerFactory(profile,
                             RTC::Create<ManagementOfAreaMap>,
                             RTC::Delete<ManagementOfAreaMap>);
  }

};


